@@ -13,11 +13,12 @@
 #  limitations under the License.
 #
 #  author: terry@odd-e.com
+#  Website: www.lizard.ws
 #
 """
 lizard is a simple code complexity analyzer without caring about the C/C++
 header files or Java imports.
-Please find the README.rst for more information.
+For more information visit http://www.lizard.ws
 """
 from __future__ import print_function
 import sys
@@ -30,7 +31,7 @@ import os
 from fnmatch import fnmatch
 import hashlib
 
-VERSION = "1.8.3"
+VERSION = "1.8.6"
 
 DEFAULT_CCN_THRESHOLD = 15
 
@@ -127,7 +128,7 @@ def create_command_line_parser(prog=None):
                         dest="sorting",
                         default=[])
 
-    parser.usage = "lizard [options] [PATH or FILE] [PATH] ... "
+    parser.usage = '''lizard [options] [PATH or FILE] [PATH] ...'''
     parser.description = __doc__
     return parser
 
@@ -199,17 +200,20 @@ class CodeInfoContext(object):
     def __init__(self, filename):
         self.fileinfo = FileInformation(filename, 0)
         self.current_line = 0
-        self.current_function = FunctionInfo('', '', 0)
         self.forgive = False
         self.newline = True
+        self.global_pseudo_function = FunctionInfo('*global*', filename, 0)
+        self.current_function = self.global_pseudo_function
 
     def add_nloc(self, count):
         self.current_function.nloc += count
         self.fileinfo.nloc += count
 
     def start_new_function(self, name):
-        self.current_function =\
-            FunctionInfo(name, self.fileinfo.filename, self.current_line)
+        self.current_function = FunctionInfo(
+            name,
+            self.fileinfo.filename,
+            self.current_line)
 
     def add_condition(self, inc=1):
         self.current_function.cyclomatic_complexity += inc
@@ -227,7 +231,7 @@ class CodeInfoContext(object):
         if not self.forgive:
             self.fileinfo.function_list.append(self.current_function)
         self.forgive = False
-        self.current_function = FunctionInfo('', '', 0)
+        self.current_function = self.global_pseudo_function
 
 
 def preprocessing(tokens, reader):
@@ -279,7 +283,7 @@ def condition_counter(tokens, reader):
         conditions = reader.conditions
     else:
         conditions = set(['if', 'for', 'while', '&&', '||', '?', 'catch',
-                         'case'])
+                          'case'])
     for token in tokens:
         if token in conditions:
             reader.context.add_condition()
@@ -322,7 +326,7 @@ class CodeReader(object):
         pass
 
     @staticmethod
-    def generate_tokens(source_code, addition=''):
+    def _generate_tokens(source_code, addition):
         # DONOT put any sub groups in the regex. Good for performance
         _until_end = r"(?:\\\n|[^\n])*"
         combined_symbals = ["||", "&&", "===", "!==", "==", "!=", "<=", ">=",
@@ -335,16 +339,36 @@ class CodeReader(object):
             r"|\"(?:\\.|[^\"])*\"" +
             r"|\'(?:\\.|[^\'])*?\'" +
             r"|//" + _until_end +
-            r"|#" + _until_end +
+            r"|\#" +
             r"|:=|::|\*\*" +
             r"|" + r"|".join(re.escape(s) for s in combined_symbals) +
+            r"|\\\n" +
             r"|\n" +
             r"|[^\S\n]+" +
             r"|.)", re.M | re.S)
-        return token_pattern.findall(source_code)
+        macro = ""
+        for token in token_pattern.findall(source_code):
+            if macro:
+                if token == "\\\n" or "\n" not in token:
+                    macro += token
+                else:
+                    yield macro
+                    yield token
+                    macro = ""
+            elif token == "#":
+                macro = token
+            else:
+                yield token
+        if macro:
+            yield macro
+
+    @staticmethod
+    def generate_tokens(source_code, addition=''):
+        return [t for t in CodeReader._generate_tokens(source_code, addition)]
 
 
 class CCppCommentsMixin(object):  # pylint: disable=R0903
+
     @staticmethod
     def get_comment_from_token(token):
         if token.startswith("/*") or token.startswith("//"):
@@ -364,6 +388,7 @@ except ImportError:
 
 
 class CLikeReader(CodeReader, CCppCommentsMixin):
+
     ''' This is the reader for C, C++ and Java. '''
 
     ext = ["c", "cpp", "cc", "mm", "cxx", "h", "hpp"]
@@ -383,6 +408,9 @@ class CLikeReader(CodeReader, CCppCommentsMixin):
                 if macro:
                     if macro.group(1) in ('if', 'ifdef', 'elif'):
                         self.context.add_condition()
+                    elif macro.group(1) == 'include':
+                        yield "#include"
+                        yield macro.group(2)
                     for _ in macro.group(2).splitlines()[1:]:
                         yield '\n'
                 else:
@@ -467,6 +495,10 @@ class CLikeReader(CodeReader, CCppCommentsMixin):
             self.context.add_to_long_function_name(" " + token)
         elif token == 'throw':
             self._state = self._state_throw
+        elif token == '(':
+            long_name = self.context.current_function.long_name
+            self.context.start_new_function(long_name)
+            self._state_function(token)
         elif token == '{':
             self.br_count += 1
             self._state = self._state_imp
@@ -517,6 +549,22 @@ class JavaReader(CLikeReader, CodeReader):
         if token == '{':
             self._state_dec_to_imp(token)
 
+    def _state_global(self, token):
+        if token == '@':
+            self._state = self._state_decorator
+            return
+        super(JavaReader, self)._state_global(token)
+
+    def _state_decorator(self, _):
+        self._state = self._state_post_decorator
+
+    def _state_post_decorator(self, token):
+        if token == '.':
+            self._state = self._state_decorator
+        else:
+            self._state = self._state_global
+            self._state(token)
+
 
 class ObjCReader(CLikeReader, CodeReader):
 
@@ -653,6 +701,7 @@ def whitelist_filter(warnings, script=None):
 
 
 class OutputScheme(object):
+
     def __init__(self, ext):
         self.extensions = ext
         self.items = [
@@ -660,7 +709,7 @@ class OutputScheme(object):
             {'caption': "  CCN  ", 'value': "cyclomatic_complexity"},
             {'caption': " token ", 'value': "token_count"},
             {'caption': " PARAM ", 'value': "parameter_count"},
-            ] + [
+        ] + [
             {
                 'caption': ext.FUNCTION_CAPTION,
                 'value': ext.FUNCTION_INFO_PART
@@ -690,7 +739,7 @@ def print_warnings(option, scheme, warnings):
     if not option.warnings_only:
         print(("\n" +
                "======================================\n" +
-              "!!!! Warnings (CCN > %d) !!!!") % option.CCN)
+               "!!!! Warnings (CCN > %d) !!!!") % option.CCN)
         print(scheme.function_info_head())
     for warning in warnings:
         warning_count += 1
@@ -704,7 +753,7 @@ def print_warnings(option, scheme, warnings):
 def print_total(warning_count, saved_result, option):
     file_infos = list(file_info for file_info in saved_result if file_info)
     all_fun = list(itertools.chain(*(file_info.function_list
-                   for file_info in file_infos)))
+                                     for file_info in file_infos)))
     cnt = len(all_fun)
     if cnt == 0:
         cnt = 1
@@ -720,8 +769,9 @@ def print_total(warning_count, saved_result, option):
         warning_count,
         float(warning_count) / cnt,
         float(sum([f.nloc for f in all_fun
-              if f.cyclomatic_complexity > option.CCN])) / nloc_in_functions
-        )
+                   if f.cyclomatic_complexity > option.CCN]))
+        / nloc_in_functions
+    )
 
     if not option.warnings_only:
         print("=" * 90)
@@ -820,8 +870,12 @@ def get_all_source_files(paths, exclude_patterns):
     hash_set = set()
 
     def _validate_file(pathname):
-        return (pathname in paths or (CodeReader.get_reader(pathname) and
-                all(not fnmatch(pathname, p) for p in exclude_patterns)
+        return (
+            pathname in paths or (
+                CodeReader.get_reader(pathname) and all(
+                    not fnmatch(
+                        pathname,
+                        p) for p in exclude_patterns)
                 and _not_duplicate(pathname)))
 
     def _not_duplicate(full_path_name):
@@ -864,13 +918,13 @@ def get_extensions(extension_names, switch_case_as_one_condition=False):
         condition_counter,
         token_counter,
         token_processor_for_function,
-        ]
+    ]
     if switch_case_as_one_condition:
         extensions.append(recount_switch_case)
 
     return extensions +\
         [import_module('lizard_ext.lizard' + name.lower()).LizardExtension()
-            if isinstance(name, str) else name for name in extension_names]
+         if isinstance(name, str) else name for name in extension_names]
 
 analyze_file = FileAnalyzer(get_extensions([]))  # pylint: disable=C0103
 
