@@ -10,19 +10,20 @@ class Test_C_Token_extension(unittest.TestCase):
         #tbd
 
 class Test_c_cpp_lizard(unittest.TestCase):
+
     def test_empty(self):
         result = get_cpp_function_list("")
         self.assertEqual(0, len(result))
-        
+
     def test_no_function(self):
         result = get_cpp_function_list("#include <stdio.h>\n")
         self.assertEqual(0, len(result))
-    
+
     def test_one_function(self):
         result = get_cpp_function_list("int fun(){}")
         self.assertEqual(1, len(result))
         self.assertEqual("fun", result[0].name)
-    
+
     def test_two_function(self):
         result = get_cpp_function_list("int fun(){}\nint fun1(){}\n")
         self.assertEqual(2, len(result))
@@ -32,7 +33,7 @@ class Test_c_cpp_lizard(unittest.TestCase):
         self.assertEqual(1, result[0].end_line)
         self.assertEqual(2, result[1].start_line)
         self.assertEqual(2, result[1].end_line)
-    
+
     def test_function_with_content(self):
         result = get_cpp_function_list("int fun(xx oo){int a; a= call(p1,p2);}")
         self.assertEqual(1, len(result))
@@ -153,7 +154,7 @@ class Test_c_cpp_lizard(unittest.TestCase):
         result = get_cpp_function_list("bool TC::operator !(int b){}")
         self.assertEqual(1, len(result))
         self.assertEqual("TC::operator !", result[0].name)
-                
+
     def test_function_operator(self):
         result = get_cpp_function_list("bool TC::operator ()(int b){}")
         self.assertEqual(1, len(result))
@@ -167,7 +168,13 @@ class Test_c_cpp_lizard(unittest.TestCase):
     def test_brakets_before_function(self):
         result = get_cpp_function_list('''()''')
         self.assertEqual(0, len(result))
-        
+
+    def test_function_that_returns_function_pointers(self):
+        result = get_cpp_function_list('''int (*fun())(){}''')
+        self.assertEqual(1, len(result))
+        self.assertEqual("int( * fun ( ) )", result[0].name)
+
+
 
 class Test_Preprocessing(unittest.TestCase):
 
@@ -178,8 +185,8 @@ class Test_Preprocessing(unittest.TestCase):
                         }
                ''')
         self.assertEqual(0, len(result))
-    
-   
+
+
     def test_preprocessors_should_be_ignored_outside_function_implementation(self):
         result = get_cpp_function_list('''
                       #ifdef MAGIC
